package japa.parser.ast.test;

import org.junit.Ignore;

@Ignore
public class ClassWithLineComments {

public void aMethod(){
    // first comment
    int a=0; //second comment
    // third comment
    // fourth comment
}

}
